/***************************************************************************
                            sWMicons.c  -  description
                            -----------------------
    begin                : Sun Jan 21 13:41:02 CEST 2001
    copyright            : (C) 2001 by Robert Sperling
    email                : sperling@small-window-manager.de
    version              : v0.4.5
 ***************************************************************************/
/*!
\file sWMicons.c
\author Robert Sperling <sperling@small-window-manager.de>
\version 0.4.5
\brief small utility to change the root window bachground

\verbatim
 ***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************
\endverbatim 
*/

#include <X11/xpm.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "../sWM-1.3.6/sWMconfiguration/config.h"

#define FILEWRITER
//#define DEBUG
//#define TRACE
#define ERRORMSG

typedef struct {
  Pixmap pixmap,mask;
  int width, height;
} sWMPixmap;


//! root window to paint in
Display                        *display;
int                             screenNo;
unsigned long                   black,white;
GC                              gc, fillgc;
Pixmap				pixmap;
Pixmap				dummyPix;
unsigned int			xmax;
unsigned int			ymax;
Window 			        rootWindow;
XpmAttributes 		        xpmAttribs;
XFontStruct                    *sWMFont;
XGCValues             	        gv;

#ifdef FILEWRITER
FILE *buttonFile[5];
#endif//FILEWRITER

// **************************************************************************

sWMPixmap loadPixmap (char *fileName)
{ sWMPixmap myPixmap;
  int    val;
#ifdef TRACE
  fprintf(stderr,"loading icon\n");
#endif//TRACE


  if ((val = XpmReadFileToPixmap (display, rootWindow, fileName, 
                                  &myPixmap.pixmap, &myPixmap.mask,
                                  &xpmAttribs))!= XpmSuccess)
  { 
#ifdef ERRORMSG  
    if(val == XpmOpenFailed)                                                  
      fprintf(stderr, "Couldn't open pixmap file\n");                         
    else if(val == XpmColorFailed)                                            
      fprintf(stderr, "Couldn't allocated required colors\n");                
    else if(val == XpmFileInvalid)                                            
      fprintf(stderr, "Invalid Format for an Xpm File\n");                    
    else if(val == XpmColorError)                                             
      fprintf(stderr, "Invalid Color specified in Xpm File\n");               
    else if(val == XpmNoMemory)                                               
      fprintf(stderr, "Insufficient Memory\n");                               
#endif//ERRORMSG

    myPixmap.pixmap=0;
  }

  myPixmap.width=xpmAttribs.width;
  myPixmap.height=xpmAttribs.height;
  
  return myPixmap;
}//loadPixmap

// **************************************************************************

/*!
\verbatim
History
-------
* 1.4.3 clickdeskx/clickdesky now repesent the size of the icons
\endverbatim
*/
void loadIcon (char* filename, int xpos, int ypos, char* iconName)
{ sWMPixmap myPixmap;
  int textSize;

  myPixmap=loadPixmap(filename);
#ifdef DEBUG  
  fprintf(stderr,"loading icon pixmap done\n");
#endif//DEBUG  
  
  
  if (myPixmap.pixmap)
  { 
#ifdef DEBUG  
    fprintf(stderr,"copying pixmap to dummy-drawable\n");
#endif//DEBUG  
  
    XCopyArea (display, myPixmap.pixmap, dummyPix, gc, 0, 0, 
               CLICKDESKX, CLICKDESKY, 
	       (xpos*CLICKDESKX)+CLICKDESKX/2-myPixmap.width/2, 
	       (ypos*CLICKDESKY)+CLICKDESKY/2-myPixmap.height/2);

#ifdef DEBUG  
    fprintf(stderr,"copying pixmap to dummy-drawable\n");
#endif//DEBUG  
	       
    XFreePixmap(display,myPixmap.pixmap);

#ifdef DEBUG  
    fprintf(stderr,"determining textsize of name: %s\n",iconName);
#endif//DEBUG  
    
    textSize=XTextWidth(sWMFont,iconName, strlen(iconName));
    
#ifdef DEBUG  
    fprintf(stderr,"textsize is: %i\n",textSize);
#endif//DEBUG  
    XFillRectangle(display, dummyPix, fillgc, 
               (xpos*CLICKDESKX)+CLICKDESKX/2-textSize/2-2,
               (ypos*CLICKDESKY)+CLICKDESKY/2+myPixmap.height/2+3, 
               textSize+2, 14);
    
    XDrawString(display,dummyPix,gc,
                (xpos*CLICKDESKX) + CLICKDESKX/2-textSize/2,
		(ypos*CLICKDESKY) + CLICKDESKY/2+myPixmap.height/2+15,
		iconName,strlen(iconName));

  }//if
}//loadIcon


// **************************************************************************


void loadBackground (char* filename, char* mode)
{ sWMPixmap background;
Pixmap dummy;
  int width,height, x,y, border, depth;
#ifdef DEBUG  
  fprintf(stderr,"loading background\n");
#endif//DEBUG  
  background=loadPixmap(filename);
  if (background.pixmap) 
  { if (!strcmp(mode,"tile"))
    { XGetGeometry (display, background.pixmap, &dummy, 
                    &x, &y, &width, &height, &border, &depth);

      for (y=0; y<ymax; y+=height)
        for (x=0; x<xmax; x+=width)
          XCopyArea (display, background.pixmap, dummyPix, gc, 0, 0, 
                     width, height, x, y);
    }	        
    XFreePixmap(display,background.pixmap);
  }
}//loadBackground


// **************************************************************************


void initFile(FILE *toInit)
{ fputs ("#!/bin/bash\n",toInit);
  fputs ("case $1-$2 in\n",toInit);
}//initFile

// **************************************************************************

void closeFile(FILE *toClose)
{ fputs ("esac\n",toClose);
  fclose (toClose);
}//closeFile

// **************************************************************************

int parserc(char *file)                                 
{ char buf[1024], *xvalue, *yvalue, *buttonValue, *toExec; 
  char *pvalue, *iconName;
  FILE *rc;                       
  int x, y, button;
  XFontStruct *dummyfont;

                                                                                         
  if((rc=fopen(file, "r")))                                                                                
  { 
#ifdef FILEWRITER  
    buttonFile[1] = fopen(".sWM/button1", "w");
    buttonFile[2] = fopen(".sWM/button2", "w");
    buttonFile[3] = fopen(".sWM/button3", "w");
    initFile(buttonFile[1]);
    initFile(buttonFile[2]);
    initFile(buttonFile[3]);    
#endif//FILEWRITER    



    while(fgets(buf, sizeof buf, rc))
    { 
#ifdef DEBUG  
  fprintf(stderr,"got a new line: %s\n",buf);
#endif//DEBUG  
     
      xvalue     = strtok(buf, ":");

#ifdef DEBUG  
  fprintf(stderr,"command (first) token is: %s\n",xvalue);
#endif//DEBUG  
      
      if(xvalue)
      { if (strcmp(xvalue,"#"))
        { if (!strcmp(xvalue,"background"))
	  { yvalue = strtok(NULL, ":");
            pvalue = strtok(NULL, "\n");
            if (yvalue && pvalue)
	      loadBackground(pvalue,yvalue);  
          }//endif xvalue=background
	  else if (!strcmp(xvalue,"iconfont"))
	  { yvalue = strtok(NULL, "\n");
#ifdef DEBUG  
            fprintf(stderr,"got iconfont\n");
            fprintf(stderr,"iconfont will be: __%s__\n",yvalue);
#endif//DEBUG  
 	    dummyfont = XLoadQueryFont(display, yvalue);

	    if (dummyfont!=(XFontStruct *) NULL)
            {  sWMFont=dummyfont;
	       gv.font = dummyfont->fid;
               XChangeGC (display, gc, GCFont, &gv);
	       fprintf(stderr,"font loaded");
	    } 

#ifdef ERRORMSG
            else fprintf(stderr,"error loading iconfont\n");
#endif//ERRORMSG  
	  }//endif xvalue=iconfont
	  else
	  { yvalue     = strtok(NULL, ":");	
            buttonValue= strtok(NULL, ":");
            pvalue     = strtok(NULL, ":\n");
            iconName   = strtok(NULL, ":");
            toExec     = strtok(NULL, "\n");
	    
#ifdef DEBUG 	    
	      fprintf (stderr,"seems we got an icon line\n");
#endif//DEBUG	      

#ifdef FILEWRITER
	    if((!strcmp(xvalue,"*")) && yvalue && buttonValue && pvalue)
	    { button= strtol(buttonValue,NULL,0);
	      fprintf (buttonFile[button],"*) %s;;\n",pvalue);
	    }  
#endif//FILEWRITER
	    
            if (yvalue && buttonValue && iconName && pvalue)
            { x     = strtol(xvalue,NULL,0);
	      y     = strtol(yvalue,NULL,0);
	      button= strtol(buttonValue,NULL,0);

#ifdef DEBUG 	    
	      fprintf (stderr,"Got line: (x=%s/y=%s/b=%s/p=%s/i=%s,2x=%s)\n",
	               xvalue,yvalue,buttonValue,pvalue,iconName,toExec);
	      fprintf (stderr,"coordinates: %i, %i\n",x,y);
#endif//DEBUG	      

              loadIcon(pvalue,x-1,y-1,iconName);
#ifdef FILEWRITER
              fprintf (buttonFile[button],"%i-%i) %s;;\n",x,y,toExec);
#endif//FILEWRITER

	    }//if all values given   
          }//if xvalue (commandType)
	}//if no comment
      }//if arg1 is valid	  
    }//endwhile                                                                                                         
  } else 
  { fprintf(stderr,"could not open file!\n");
    return -1;
  }
  fclose(rc);
#ifdef FILEWRITER
    closeFile(buttonFile[1]);
    closeFile(buttonFile[2]);
    closeFile(buttonFile[3]);    
#endif//FILEWRITER    
  return 0;
}//parserc






int main(int argc, char *argv[])
{ // Variableninitialisierung
  XWindowAttributes 		rootAttribs;                                                  
    
  // Display oeffnen 
  display = XOpenDisplay(""); 
  if (display) 
  { rootWindow = RootWindow (display, screenNo);
    xmax = DisplayWidth (display, screenNo);                                                                     
    ymax = DisplayHeight(display, screenNo);
    dummyPix = XCreatePixmap(display,rootWindow,xmax,ymax,DefaultDepth(display,screenNo));
 
    //Vorder & Hintergrund festlegen
    XGetWindowAttributes(display,rootWindow,&rootAttribs);                                    
    xpmAttribs.colormap = rootAttribs.colormap;             
    xpmAttribs.valuemask = XpmSize | XpmReturnPixels | XpmColormap;


    gv.function = GXinvert;
    gv.fill_style=FillStippled;
    fillgc      = XCreateGC (display, rootWindow, GCFunction|GCFillStyle, &gv);                                   
    
    gv.function = GXcopy;
    gv.foreground= BlackPixel (display, DefaultScreen (display));
    sWMFont = XLoadQueryFont(display, "fixed");
    gv.font     = sWMFont->fid;
    gc          = XCreateGC (display, rootWindow, GCFunction, &gv);

  
    if (argc==2 && argv[1])
      loadBackground (argv[1], "tile");

#ifdef DEBUG   
    fprintf(stderr,"parsing\n");
#endif//DEBUG    
    if (parserc(".sWM/sWMiconsrc")==0)
    {
#ifdef DEBUG 
      fprintf(stderr,"parsing done\n");
#endif//DEBUG
      system ("chmod u+x .sWM/button*");
    
      XSetWindowBackgroundPixmap(display, rootWindow, dummyPix);
      XClearWindow(display,rootWindow);
      XSetCloseDownMode(display, RetainPermanent); 
    }//if parserc
    
    XFreeFont(display,sWMFont);
    XFreeGC(display,gc);
    XFreeGC(display,fillgc);
    XCloseDisplay(display);
    return 0;
  }
  return 1;
}//main
